#include "BSL_UART_Protocol_5xx.h"
#include "BSL_USB_Protocol_5xx.h"
#include "BSL_UART_Protocol_ROM.h"
#include "BSL_IO_UART.h"
#include "BSL_IO_USB.h"
#include "BSL_Commands_5xx.h"
#include "BSL_Commands_ROM.h"
#include "BSL_Commands_P4xx.h"
#include "BSL_Interface.h"
#include "TextFileIO.h"
#include "BSL_Definitions.h"
#include <string.h>

void (*BSL_set_TX_Packet)( unsigned char (*txFunc)(dataBuffer db));
void (*BSL_set_RX_Packet)( unsigned char (*rxFunc)(dataBuffer *db));

int BSL_Version;
unsigned int BSL_bufferSize;
unsigned char versionString[60];

/*******************************************************************************
*Function:    BSL_TX_BSL_Version_String
*Description: fills versionString with values
*Parameters:
*             none
*Returns:
*             unsigned char *       pointer to versionString
*DataBuffer:
*             [8]                   Vendor[0]
*             [9]                   Vendor[1]
*            [16]                   CI[0]
*            [17]                   CI[1]
*            [25]                   API[0]
*            [26]                   API[1]
*            [33]                   PI[0]
*            [34]                   PI[1]
*******************************************************************************/
unsigned char* BSL_TX_BSL_Version_String()
{
	dataBuffer rx_buf;
	unsigned char success = 0xFF;
	strncpy(versionString, "0", 36);

	success = BSL_TX_BSL_Version();
	if (success == BSL_ACK)
	{
		rx_buf = BSL_get_RX_Buffer();
		if ((BSL_Version == FAMILY_5438) || (BSL_Version == FAMILY_FLASH) || (BSL_Version == FAMILY_FRAM))
		{
			strncpy(versionString, "Vendor:[xx],CI:[xx],API:[xx],PI:[xx]", 36);
			if (rx_buf.data[1] == 0x00)
			{
				versionString[8] = 'T';
				versionString[9] = 'I';
			}
			else
			{
				// Vend
				versionString[8] = ((rx_buf.data[1] >> 4) & 0x0F) + '0';
				if (((rx_buf.data[1] >> 4) & 0x0F) > 9)
				{
					versionString[8] = versionString[8] + 7;  // number is in hex format, you need to add 7 here to get 'A' instead of ':'
				}
				versionString[9] = ((rx_buf.data[1]) & 0x0F) + '0';
				if (((rx_buf.data[1]) & 0x0F) > 9)
				{
					versionString[9] = versionString[9] + 7;
				}
			}
			// CI
			versionString[16] = ((rx_buf.data[2] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[2] >> 4) & 0x0F) > 9)
			{
				versionString[16] = versionString[16] + 7;
			}
			versionString[17] = ((rx_buf.data[2]) & 0x0F) + '0';
			if (((rx_buf.data[2]) & 0x0F) > 9)
			{
				versionString[17] = versionString[17] + 7;
			}
			// API
			versionString[25] = ((rx_buf.data[3] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[3] >> 4) & 0x0F) > 9)
			{
				versionString[25] = versionString[25] + 7;
			}
			versionString[26] = ((rx_buf.data[3]) & 0x0F) + '0';
			if (((rx_buf.data[3]) & 0x0F) > 9)
			{
				versionString[26] = versionString[26] + 7;
			}
			// PI
			versionString[33] = ((rx_buf.data[4] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[4] >> 4) & 0x0F) > 9)
			{
				versionString[33] = versionString[33] + 7;
			}
			versionString[34] = ((rx_buf.data[4]) & 0x0F) + '0';
			if (((rx_buf.data[4]) & 0x0F) > 9)
			{
				versionString[34] = versionString[34] + 7;
			}
		}
		else if ((BSL_Version == FAMILY_ROM) || (BSL_Version == FAMILY_MSP432))
		{
			strncpy(versionString, "ERROR", 5);
		}
	}
	else
	{
		strncpy(versionString, "ERROR", 5);
	}
	return versionString;
}

/*******************************************************************************
*Function:    BSL_TX_BSL_Version_String_32
*Description: fills versionString with values
*Parameters:
*             none
*Returns:
*             unsigned char *       pointer to versionString
*DataBuffer:
*             [8]                   Vendor[0]
*             [9]                   Vendor[1]
*            [10]                   Vendor[2]
*            [11]                   Vendor[3]
*            [18]                   CI[0]
*            [19]                   CI[1]
*            [20]                   CI[2]
*            [21]                   CI[3]
*            [29]                   API[0]
*            [30]                   API[1]
*            [31]                   API[2]
*            [32]                   API[3]
*            [39]                   PI[0]
*            [40]                   PI[1]
*            [42]                   Build ID[0]
*            [43]                   Build ID[1]
*******************************************************************************/
unsigned char* BSL_TX_BSL_Version_String_32()
{
	dataBuffer rx_buf;
	unsigned char success = 0xFF;
	strncpy(versionString, "", 45);
	success = BSL_TX_BSL_Version_32();
	if (success == BSL_ACK)
	{
		rx_buf = BSL_get_RX_Buffer();
		if (BSL_Version == FAMILY_MSP432)
		{
			strncpy(versionString, "Vendor:[xxxx],CI:[xxxx],API:[xxxx],PI:[xxxx],BID:[xxxx]", 45);
/***BSL Vendor*****************************************************/
			if ((rx_buf.data[1] == 0) && (rx_buf.data[2] == 0))
			{
				versionString[8]  = ' ';
				versionString[9]  = 'T';
				versionString[10] = 'I';
				versionString[11] = ' ';
			}
			else
			{
				versionString[8] = ((rx_buf.data[1] >> 4) & 0x0F) + '0';
				if (((rx_buf.data[1] >> 4) & 0x0F) > 9)
				{
					versionString[8] = versionString[8] + 7;  // number is in hex format, you need to add 7 here to get 'A' instead of ':'
				}
				versionString[9] = (rx_buf.data[1] & 0x0F) + '0';
				if ((rx_buf.data[1] & 0x0F) > 9)
				{
					versionString[9] = versionString[9] + 7;
				}
				versionString[10] = ((rx_buf.data[2] >> 4) & 0x0F) + '0';
				if (((rx_buf.data[2] >> 4) & 0x0F) > 9)
				{
					versionString[10] = versionString[10] + 7;
				}
				versionString[11] = (rx_buf.data[2] & 0x0F) + '0';
				if ((rx_buf.data[2] & 0x0F) > 9)
				{
					versionString[11] = versionString[11] + 7;
				}
			}
/***Command Interpreter*********************************************/
			versionString[18] = ((rx_buf.data[3] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[3] >> 4) & 0x0F) > 9)
			{
				versionString[18] = versionString[18] + 7;
			}
			versionString[19] = (rx_buf.data[3] & 0x0F) + '0';
			if ((rx_buf.data[3] & 0x0F) > 9)
			{
				versionString[19] = versionString[19] + 7;
			}
			versionString[20] = ((rx_buf.data[4] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[4] >> 4) & 0x0F) > 9)
			{
				versionString[20] = versionString[20] + 7;
			}
			versionString[21] = (rx_buf.data[4] & 0x0F) + '0';
			if ((rx_buf.data[4] & 0x0F) > 9)
			{
				versionString[21] = versionString[21] + 7;
			}
/***API**************************************************************/
			versionString[29] = ((rx_buf.data[5] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[5] >> 4) & 0x0F) > 9)
			{
				versionString[29] = versionString[29] + 7;
			}
			versionString[30] = ((rx_buf.data[5]) & 0x0F) + '0';
			if ((rx_buf.data[5] & 0x0F) > 9)
			{
				versionString[30] = versionString[30] + 7;
			}
			versionString[31] = ((rx_buf.data[6] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[6] >> 4) & 0x0F) > 9)
			{
				versionString[31] = versionString[31] + 7;
			}
			versionString[32] = ((rx_buf.data[6]) & 0x0F) + '0';
			if ((rx_buf.data[6] & 0x0F) > 9)
			{
				versionString[32] = versionString[32] + 7;
			}
/***Peripheral Intetface*************************************/
			versionString[39] = ((rx_buf.data[7] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[7] >> 4) & 0x0F) > 9)
			{
				versionString[39] = versionString[39] + 7;
			}
			versionString[40] = (rx_buf.data[7] & 0x0F) + '0';
			if ((rx_buf.data[7] & 0x0F) > 9)
			{
				versionString[40] = versionString[40] + 7;
			}
			versionString[41] = ((rx_buf.data[8] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[8] >> 4) & 0x0F) > 9)
			{
				versionString[41] = versionString[41] + 7;
			}
			versionString[42] = (rx_buf.data[8] & 0x0F) + '0';
			if ((rx_buf.data[8] & 0x0F) > 9)
			{
				versionString[42] = versionString[42] + 7;
			}
/***Build ID**************************************************/
			versionString[50] = ((rx_buf.data[9] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[9] >> 4) & 0x0F) > 9)
			{
				versionString[50] = versionString[50] + 7;
			}
			versionString[51] = (rx_buf.data[9] & 0x0F) + '0';
			if ((rx_buf.data[9] & 0x0F) > 9)
			{
				versionString[51] = versionString[51] + 7;
			}
			versionString[52] = ((rx_buf.data[10] >> 4) & 0x0F) + '0';
			if (((rx_buf.data[10] >> 4) & 0x0F) > 9)
			{
				versionString[52] = versionString[52] + 7;
			}
			versionString[53] = (rx_buf.data[10] & 0x0F) + '0';
			if ((rx_buf.data[10] & 0x0F) > 9)
			{
				versionString[53] = versionString[53] + 7;
			}
		}
	}
	else
	{
		strncpy(versionString, "ERROR", 5);
	}
	return versionString;
}


/*******************************************************************************
*Function:    BSL_TX_BufferSize
*Description: return size of BSL_TX_Buffer
*Parameters:
*             none
*Returns:
*             signed int	size of buffer
*******************************************************************************/
signed int BSL_TX_BufferSize()
{
	if ((BSL_Version == FAMILY_5438) || (BSL_Version == FAMILY_FLASH) || (BSL_Version == FAMILY_FRAM) || (BSL_Version == FAMILY_MSP432))
	{
		if (Flash_TX_BufferSize() == BSL_ACK)
		{
			dataBuffer rx_buf;
			signed int retValue;
			rx_buf = BSL_get_RX_Buffer();
			retValue = rx_buf.data[1];
			retValue |= rx_buf.data[2] << 8;
			return retValue;
		}
		else
		{
			return BSL_NACK;
		}
	}
	else
	{
		return WRONG_FAMILY; // wrong family
	}
}

/*******************************************************************************
*Function:    BSL_TX_TXT_File
*Description: reads data to TI TXT file
*Parameters:
*             char *        fileName	file name
*             unsigned int  addr		address where code needs to be placed
*             unsigned int length		length of the block
*Returns:
*             unsigned int				BSL_ACK if operation was successfull
*******************************************************************************/
// Read from BSL to text file
unsigned int BSL_TX_TXT_File(char* fileName, unsigned int addr, unsigned int length, unsigned char word_length)
{
	unsigned char buf[(1024*512)];
	DWORD start, stop;
	float seconds, kbs;
	unsigned int answer;

	// below is the better way to do things, but crashes currently
	//unsigned char *buf = malloc((size_t)length);

	openTI_TextForWrite(fileName);
	start = GetTickCount();
	if (BSL_SetVerbose == &USB_setVerbose)
	{
		if (word_length == WORD_ADDRESSED)
		{
			answer = BSL_TX_DataBlock_32(addr, length, buf);
			if (answer == BSL_ACK)
			{
				writeTI_TextFile(addr, buf, length);
			}
			else
			{
				return answer;
			}
		}
		else
		{
			answer = BSL_TX_DataBlock(addr, length, buf);
			if (answer == BSL_ACK)
			{
				writeTI_TextFile(addr, buf, length);
			}
			else
			{
				return answer;
			}
		}
	}
	else
	{
		unsigned int curLength = BSL_bufferSize;
		unsigned int curAddr;
		for( curAddr = addr; curAddr < (addr + length); curAddr += curLength)
		{
			if ((addr + length) - curAddr < BSL_bufferSize)
			{
				curLength = (addr + length) - curAddr;
			}
			if (word_length == WORD_ADDRESSED)
			{
				answer = BSL_TX_DataBlock_32(curAddr, curLength, &buf[curAddr - addr]);
			}
			else
			{
				answer = BSL_TX_DataBlock(curAddr, curLength, &buf[curAddr - addr]);
			}
			if (answer == BSL_ACK )
			{
			}
			else
			{
				return answer;
			}
		}
		writeTI_TextFile(addr, buf, length); 
	}
	stop = GetTickCount();
	seconds = ((float)stop - (float)start) / (float)1000;
	kbs = ((float)length / (float)1024) / (float)seconds;
	printf("    Read %i bytes in %.2f seconds [%.2f Kbytes/s]\n", length, seconds, kbs);
	endTI_TextWrite();
	return BSL_ACK;
}

/*******************************************************************************
*Function:    BSL_RX_TXT_File
*Description: writes data from TI TXT file to memory
*Parameters:
*             char *         fileName    file name
*             unsigned char  fast        determines if the return value of BSL_TX_DataBlock is evaluated
*Returns:
*             unsigned int               BSL_ACK if operation was successfull
*DataBuffer:
*             [1]
*             [2]
*******************************************************************************/
unsigned int BSL_RX_TXT_File(char* fileName, unsigned char fast, unsigned char word_length)
{
	DWORD start, stop;
	float seconds, kbs;
	DataBlock data;
	unsigned int numBytes=0;
	openTI_TextForRead(fileName);
	start = GetTickCount();
	unsigned int answer;
	while (moreDataToRead())
	{
		data = readTI_TextFile(BSL_bufferSize);
		numBytes += data.numberOfBytes;
		if(!fast)
		{
			if (word_length == WORD_ADDRESSED)
			{
				answer = BSL_RX_DataBlock_32(data);
				if (answer != BSL_ACK)
				{
					answer = BSL_RX_DataBlock_32(data);
					if (answer != BSL_ACK)
					{
						return answer;
					}
				}
			}
			else
			{
				answer = BSL_RX_DataBlock(data);
				if (answer != BSL_ACK)
				{
					answer = BSL_RX_DataBlock(data);
					if (answer != BSL_ACK)
					{
						return answer;
					}
				}
			}
		}
		else
		{
			BSL_RX_DataBlock_Fast(data);
		}
	}
	stop = GetTickCount();
	seconds = ((float)stop - (float)start) / (float)1000;
	kbs = ((float)numBytes / (float)1024) / (float)seconds;
	printf("    Wrote %i bytes in %.2f seconds [%.2f Kbytes/s]\n", numBytes, seconds, kbs);
	return BSL_ACK;
}

/*******************************************************************************
*Function:    BSL_setFamily
*Description: sets family-dependent values of device
*Parameters:
*             unsigned int family
*Returns:
*             none
*******************************************************************************/
void BSL_setFamily(unsigned int family)
{
	if (family == FAMILY_ROM)
	{
		BSL_RX_Password = &ROM_RX_Password;
		BSL_RX_DataBlock = &ROM_RX_DataBlock;
		BSL_TX_DataBlock = &ROM_TX_DataBlock;
		BSL_eraseSegment = &ROM_eraseSegment;
		BSL_massErase = &ROM_massErase;
		BSL_eraseCheck = &ROM_eraseCheck;
		BSL_setMemOffset = &ROM_setMemOffset;
		BSL_set_TX_Packet = &ROM_set_TX_Packet;
		BSL_set_RX_Packet = &ROM_set_RX_Packet;
		BSL_get_RX_Buffer = &ROM_get_RX_Buffer;
		BSL_eraseMainOrInfo = &ROM_eraseMainOrInfo;
		BSL_Version = FAMILY_ROM;
		BSL_bufferSize = 240;
		BSL_TX_BSL_Version_32 = &BSL_unknownCommand;
		BSL_TX_DataBlock_32 = &BSL_unknownCommand;
		BSL_LoadPC_32 = &BSL_unknownCommand;
		BSL_CRC_Check_32 = &BSL_unknownCommand;
		BSL_eraseSegment_32 = &BSL_unknownCommand;
		BSL_RX_DataBlock_32 = &BSL_unknownCommand;
		BSL_RX_Password_32 = &BSL_unknownCommand;
		BSL_rebootReset = &BSL_unknownCommand;
	}
	else if ((family == FAMILY_FLASH) || (family == FAMILY_5438) || (family == FAMILY_FRAM))
	{
		BSL_set_TX_Packet = &Flash_set_TX_Packet;
		BSL_set_RX_Packet = &Flash_set_RX_Packet;
		BSL_TX_BSL_Version = &Flash_TX_BSL_Version;
		BSL_TX_DataBlock = &Flash_TX_DataBlock;
		BSL_LoadPC = &Flash_LoadPC;
		BSL_CRC_Check = &Flash_CRC_Check;
		BSL_massErase = &Flash_massErase;
		BSL_toggleInfo = &Flash_toggleInfo;
		BSL_eraseSegment = &Flash_eraseSegment;
		BSL_RX_Password = &Flash_RX_Password;
		BSL_RX_DataBlock = &Flash_RX_DataBlock;
		BSL_RX_DataBlock_Fast = &Flash_RX_DataBlock_Fast;
		BSL_get_RX_Buffer = &Flash_get_RX_Buffer;
		BSL_changeBaudRate = &Flash_changeBaudRate;
		BSL_bufferSize = 240;
		BSL_Version = FAMILY_FLASH;
		BSL_TX_BSL_Version_32 = &BSL_unknownCommand;
		BSL_TX_DataBlock_32 = &BSL_unknownCommand;
		BSL_LoadPC_32 = &BSL_unknownCommand;
		BSL_CRC_Check_32 = &BSL_unknownCommand;
		BSL_eraseSegment_32 = &BSL_unknownCommand;
		BSL_RX_DataBlock_32 = &BSL_unknownCommand;
		BSL_RX_Password_32 = &BSL_unknownCommand;
		BSL_rebootReset = &BSL_unknownCommand;
		BSL_eraseMainOrInfo = &BSL_unknownCommand;
		BSL_setMemOffset = &BSL_unknownCommand;
	}
	else if (family == FAMILY_MSP432)
	{
		BSL_set_TX_Packet = &Flash_set_TX_Packet_32;
		BSL_set_RX_Packet = &Flash_set_RX_Packet_32;
		BSL_TX_BSL_Version_32 = &Flash_TX_BSL_Version_32;
		BSL_TX_BSL_Version = &Flash_TX_BSL_Version;
		BSL_TX_DataBlock_32 = &Flash_TX_DataBlock_32;
		BSL_TX_DataBlock = &Flash_TX_DataBlock;
		BSL_LoadPC_32 = &Flash_LoadPC_32;
		BSL_LoadPC = &Flash_LoadPC;
		BSL_CRC_Check_32 = &Flash_CRC_Check_32;
		BSL_CRC_Check = &Flash_CRC_Check;
		BSL_massErase = &Flash_massErase;
		BSL_rebootReset = &Flash_rebootReset;
		BSL_toggleInfo = &Flash_toggleInfo;
		BSL_eraseSegment_32 = &Flash_eraseSegment_32;
		BSL_eraseSegment = &Flash_eraseSegment;
		BSL_RX_Password_32 = &Flash_RX_Password_32;
		BSL_RX_Password = &Flash_RX_Password;
		BSL_RX_DataBlock_32 = &Flash_RX_DataBlock_32;
		BSL_RX_DataBlock = &Flash_RX_DataBlock;
		BSL_RX_DataBlock_Fast = &Flash_RX_DataBlock_Fast;
		BSL_get_RX_Buffer = &Flash_get_RX_Buffer_32;
		BSL_changeBaudRate = &Flash_changeBaudRate;
		BSL_bufferSize = 240;
		BSL_Version = FAMILY_MSP432;
		BSL_eraseMainOrInfo = &BSL_unknownCommand;
		BSL_setMemOffset = &BSL_unknownCommand;
	}
	if (family == FAMILY_5438)
	{
		BSL_RX_Password = &Flash_RX_Password_5438;
		BSL_Version = FAMILY_5438;
	}
	if (family == FAMILY_FRAM)
	{
		BSL_Version = FAMILY_FRAM;
	}
}

/*******************************************************************************
*Function:    BSL_setCom
*Description: sets com-dependent values
*Parameters:
*             unsigned int com
*Returns:
*             none
*******************************************************************************/
void BSL_setCom( unsigned int com )
{
	if((com == UART) || (com == I2C) || (com == SPI))
	{
		BSL_SetVerbose = &UART_setVerbose;
		BSL_initialize_BSL = &UART_initialize_BSL;
	
		if (BSL_Version == FAMILY_ROM)
		{
			BSL_set_TX_Packet(&UART_ROM_TXPacket);
			BSL_set_RX_Packet(&UART_ROM_RXPacket);
		}
		else if ((BSL_Version == FAMILY_5438) || (BSL_Version == FAMILY_FLASH) || (BSL_Version == FAMILY_MSP432) || (BSL_Version == FAMILY_FRAM))
		{
			BSL_set_TX_Packet(&UART_5xx_TXPacket);
			BSL_set_RX_Packet(&UART_5xx_RXPacket);
			if( BSL_Version == FAMILY_5438)
			{
				BSL_initialize_BSL = &UART_initialize_BSL_5438;
			}
		}
	}
	else if (com == USB)
	{
		BSL_SetVerbose = &USB_setVerbose;
		BSL_initialize_BSL = &USB_initialize_BSL;
		BSL_set_TX_Packet(&USB_5xx_TXPacket);
		BSL_set_RX_Packet(&USB_5xx_RXPacket);
		BSL_bufferSize = 48;
	}
}

/*******************************************************************************
*Function:     BSL_setUpConnection
*Description:  This function sets up the connection, the protocol and the baudrate
*Parameters:
*              unsigned char *token
*              unsigned char protocol
*              long          rate
*Returns:
*              none
*******************************************************************************/
void BSL_setUpConnection(unsigned char *token, unsigned char protocol, long rate, unsigned char family)
{
	char pre_Com[] = "\\\\.\\";
	char* final_com;
	unsigned char reply;

	if (strlen(token) > 4)
	{
		final_com = malloc(strlen(token) + strlen(pre_Com));
		strncpy(final_com, pre_Com, 7);
		strcat(final_com, token);
	}
	else
	{
		final_com = &token[0];
	}
	BSL_setCom(protocol);
	switch (protocol)
	{
	case UART:
		printf("UART %u ", rate);
		break;
	case USB:
		printf("USB ");
		break;
	case I2C:
		printf("I2C %u ", rate);
		break;
	case SPI:
		printf("SPI %u ", rate);
		break;
	default:
		printf("Unknown protocol");
		break;
	}
	printf(" %s", token);
	reply = BSL_initialize_BSL(final_com, rate, protocol, family);
	if (reply == BSL_ACK)
	{
		printf("\tDONE");
	}
	else
	{
		printf("\tFAIL(0x%X)", reply);
	}
}

/*****************************************************************
*Function:     BSL_unknownCommand()
*Description:  This function returns 0x07(unknown Command) 
               for functions not supported by the device family
*Parameters:
*              none
*Returns:
*              unsigned char      0x07 -> unknown Command
*****************************************************************/
unsigned char BSL_unknownCommand()
{
	return UNKNOWN_COMMAND;
}