#include "BSL_Definitions.h"

#define FAMILY_5438   1
#define FAMILY_ROM    2
#define FAMILY_FLASH  3
#define FAMILY_FRAM   4
#define FAMILY_MSP432 5

#define UART 1
#define USB  2
#define I2C  3
#define SPI  4

void BSL_setFamily(unsigned int family);
void BSL_setCom(unsigned int com);

unsigned int BSL_TX_TXT_File( char* fileName, unsigned int addr, unsigned int length, unsigned char word_length);
unsigned int BSL_RX_TXT_File( char* fileName, unsigned char fast, unsigned char word_length);
void BSL_setUpConnection(unsigned char *token, unsigned char protocol, long rate, unsigned char family);
signed int BSL_TX_BufferSize();
unsigned char* BSL_TX_BSL_Version_String();
unsigned char* BSL_TX_BSL_Version_String_32();

dataBuffer (*BSL_get_RX_Buffer)();
unsigned char (*BSL_initialize_BSL)( unsigned char* com, long rate, unsigned int mode, unsigned char family);
void (*BSL_SetVerbose)( unsigned int verb ); 
unsigned char (*BSL_TX_BSL_Version)();
unsigned char(*BSL_TX_BSL_Version_32)();
unsigned char (*BSL_TX_DataBlock)(unsigned long int addr, unsigned long int length, unsigned char *buffer);
unsigned char(*BSL_TX_DataBlock_32)(unsigned long int addr, unsigned long int length, unsigned char *buffer);
unsigned char (*BSL_eraseCheck)(unsigned long int addr, unsigned long int length);
unsigned char (*BSL_LoadPC)(unsigned long int addr);
unsigned char(*BSL_LoadPC_32)(unsigned long int addr);
unsigned char (*BSL_setMemOffset)(unsigned long int addr);
unsigned char (*BSL_CRC_Check)(unsigned long int addr, unsigned long int length);
unsigned char(*BSL_CRC_Check_32)(unsigned long int addr, unsigned long int length);
unsigned char (*BSL_massErase)(unsigned char family);
unsigned char (*BSL_rebootReset)();
unsigned char (*BSL_toggleInfo)();
unsigned char (*BSL_eraseSegment)(unsigned long int addr);
unsigned char(*BSL_eraseSegment_32)(unsigned long int addr);
unsigned char (*BSL_eraseMainOrInfo)(unsigned long int addr);
unsigned char (*BSL_RX_Password)(DataBlock data);
unsigned char(*BSL_RX_Password_32)(DataBlock data);
unsigned char (*BSL_RX_DataBlock)(DataBlock data);
unsigned char(*BSL_RX_DataBlock_32)(DataBlock data);
unsigned char (*BSL_RX_DataBlock_Fast)(DataBlock data);
unsigned char (*BSL_changeBaudRate)(unsigned int rate);
unsigned char BSL_unknownCommand();

static DataBlock default_pass = {
	{(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, 
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, 
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF,
	(char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF, (char)0xFF},
	256,
	0xFFE0
};